----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_fifo_double_register.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Small fifo built with two registers.
--
-- Last update: 2020-11-30
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

use work.axi4_pkg.all;
use work.scalp_misc.all;

entity scalp_fifo_double_register is

    port (
        -- System Clock and Reset
        SysClkxCI      : in  std_ulogic;
        SysRstxRNAI    : in  std_ulogic;
        -- Axis Ports (Slave Side)
        Axism2sLinkxDI : in  t_axi4m2s;
        Axiss2mLinkxDO : out t_axi4s2m;
        -- Axis Ports (Master Side)
        Axism2sLinkxDO : out t_axi4m2s;
        Axiss2mLinkxDI : in  t_axi4s2m);

end entity scalp_fifo_double_register;

architecture behavioral of scalp_fifo_double_register is

    -- Signals
    -- Axis Data, Valid, Last, Keep
    signal Axism2s0xD  : t_axi4m2s := C_NO_AXI4_M2S;
    signal Axism2s1xDP : t_axi4m2s := C_NO_AXI4_M2S;
    signal Axism2s1xDN : t_axi4m2s := C_NO_AXI4_M2S;
    signal Axism2s2xDP : t_axi4m2s := C_NO_AXI4_M2S;
    signal Axism2s2xDN : t_axi4m2s := C_NO_AXI4_M2S;
    signal Axism2s3xD  : t_axi4m2s := C_NO_AXI4_M2S;
    -- Axis Ready
    signal Axiss2m0xD  : t_axi4s2m := C_NO_AXI4_S2M;
    signal Axiss2m1xDP : t_axi4s2m := C_NO_AXI4_S2M;
    signal Axiss2m1xDN : t_axi4s2m := C_NO_AXI4_S2M;

begin  -- architecture behavioral

    EntityIOxB : block is
    begin  -- block EntityIOxB
        -- Axis m2s
        Axism2s0FromxAS : Axism2s0xD     <= Axism2sLinkxDI;
        Axism2s3ToxAS   : Axism2sLinkxDO <= Axism2s3xD;
        -- Axis s2m
        Axiss2m0FromxAS : Axiss2m0xD     <= Axiss2mLinkxDI;
        Axiss2m1FromxAS : Axiss2m1xDN    <= Axiss2m0xD;
        Axiss2m1ToxAS   : Axiss2mLinkxDO <= Axiss2m1xDP;
    end block EntityIOxB;

    AsyncStatementsxB : block is
    begin  -- block AsyncStatementsxB        
        Axism2s3FromxAS : Axism2s3xD <= Axism2s1xDP when
                                        (Axiss2m0xD.ReadyxS = '0' and Axiss2m1xDP.ReadyxS = '1') or
                                        (Axiss2m0xD.ReadyxS = '1' and Axiss2m1xDP.ReadyxS = '1') else
                                        Axism2s2xDP;
    end block AsyncStatementsxB;

    UpdateRegxP : process (SysClkxCI, SysRstxRNAI) is
    begin  -- process UpdateRegxP
        if SysRstxRNAI = '0' then
            -- Axis m2s
            Axism2s1xDP <= C_NO_AXI4_M2S;
            Axism2s2xDP <= C_NO_AXI4_M2S;
            -- Axis s2m
            Axiss2m1xDP <= C_NO_AXI4_S2M;
        -- Enable Port
        -- ScalpFifoEnablexSP <= '0';
        elsif rising_edge(SysClkxCI) then
            -- Axis m2s
            Axism2s1xDP <= Axism2s1xDN;
            Axism2s2xDP <= Axism2s2xDN;
            -- Axis s2m
            Axiss2m1xDP <= Axiss2m1xDN;
        -- Enable Port
        -- ScalpFifoEnablexSP <= ScalpFifoEnablexSN;
        end if;
    end process UpdateRegxP;

    TransistionOnReadyxP : process (Axism2s0xD, Axism2s1xDP, Axism2s2xDP,
                                    Axiss2m0xD.ReadyxS, Axiss2m1xDP.ReadyxS) is
    begin  -- process TransistionOnReadyxP
        -- Defaults values
        Axism2s1xDN <= Axism2s1xDP;
        Axism2s2xDN <= Axism2s2xDP;

        if Axiss2m0xD.ReadyxS = '0' and Axiss2m1xDP.ReadyxS = '0' then
            Axism2s1xDN <= Axism2s1xDP;
            Axism2s2xDN <= Axism2s2xDP;
        elsif Axiss2m0xD.ReadyxS = '0' and Axiss2m1xDP.ReadyxS = '1' then
            Axism2s1xDN <= Axism2s0xD;
            Axism2s2xDN <= Axism2s2xDP;
        elsif Axiss2m0xD.ReadyxS = '1' and Axiss2m1xDP.ReadyxS = '0' then
            Axism2s1xDN <= Axism2s1xDP;
            Axism2s2xDN <= Axism2s0xD;
        else
            Axism2s1xDN <= Axism2s0xD;
            Axism2s2xDN <= Axism2s0xD;
        end if;
    end process TransistionOnReadyxP;

end architecture behavioral;
